/* Display the Rosenbrock function using polygons.
   Francois faure, UJF-Grenoble, 2003
*/

#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "traqueboule.h"

const int W_fen = 600;
const int H_fen = 600;


// Tabulated Rosenbrock function
const int w_img = 10;
const int h_img = 10;
GLfloat coords[h_img*w_img*3]; // points (x,y,z)
GLfloat norm[h_img*w_img*3]; // normals (x,y,z)
GLuint indices[2*(w_img-1)*h_img];     // indices for TRIANGLE_STRIP
const float xmin = -1.0, xmax=1.0, ymin=-1.0, ymax = 1.0;
float zMax = 0;

// material
GLfloat m_emission[] =  {0, 0, 0};
GLfloat m_ambient[] =   {1,1,1};
GLfloat m_diffuse[] =   {0.5,0.5,0.5};
GLfloat m_specular[] =  {0.5,0.5,0.5};
GLfloat m_shininess =   20.0;

// global lighting parameters
GLfloat l_ambient[] = {0.1,0.1,0.1};
GLint l_localViewer = GL_TRUE;
GLint l_twoSides = GL_TRUE;
GLint l_separateDiffuseAndSpecular = GL_SINGLE_COLOR;

// light source 0
bool lumiere0 = true;
GLfloat l0_position[] = {0,0,0,1};
GLfloat l0_ambient[] = {0.1,0.1,0.1};
GLfloat l0_diffuse[] = {1,0,0};
GLfloat l0_specular[] = {1,0,0};

// light source 1
bool lumiere1 = true;
GLfloat l1_position[] = {0,0,0,1};
GLfloat l1_ambient[] = {0.1,0.1,0.1};
GLfloat l1_diffuse[] = {1,0,0};
GLfloat l1_specular[] = {0,1,0};
GLfloat lattitude1 = 0;
GLfloat longitude1 = 0;
GLfloat distance1 = 2;


void computeGeometry()
{
	int index=0;
	for( int i=0; i<w_img; ++i ) 
		for( int j=0; j<h_img; ++j )
		{  
			float x = xmin +  i * (xmax-xmin)/(w_img-1) ; 
			float y = ymin +  j * (ymax-ymin)/(h_img-1) ;
            // Rosenbrock
			float z = 100*(y-x*x)*(y-x*x) + (1-x)*(1-x);

            // vertex coords
            coords[index  ] = x;
            coords[index+1] = y;
            coords[index+2] = z;
			
            // normals
			float dzdx = -400*x*(y-x*x) - 2*(1-x);
			float dzdy = 200*(y-x*x);
			norm[index  ]=-dzdx;
			norm[index+1]=-dzdy;
			norm[index+2]=1;
			
            //printf("normal: %f %f %f\n",norm[index], norm[index+1], norm[index+2]);
			 
			index+=3;
			
			if(z>zMax) zMax=z;
			//printf("%f %f %f\n",x,y,z);
		}
	
    // indices for TRIANGLE_STRIP
	index=0;
	for(int i=1; i<w_img; i++){
		for(int j=0; j<h_img; ++j)
		{
			indices[index++]= (i-1)*h_img+j;
			indices[index++]= (i  )*h_img+j;
		}
	}
	
	glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3,GL_FLOAT,0,coords);
	glEnableClientState(GL_NORMAL_ARRAY);
	glNormalPointer(GL_FLOAT,0,norm);
}

void display(void)
{
    glEnable( GL_DEPTH_TEST );
	glPolygonMode(GL_FRONT,GL_FILL);
	glPolygonMode(GL_BACK,GL_LINE);
	glEnable(GL_LIGHTING);
	if( lumiere0 ) glEnable(GL_LIGHT0); else glDisable(GL_LIGHT0);
	if( lumiere1 ) glEnable(GL_LIGHT1); else glDisable(GL_LIGHT1);
	glEnable(GL_NORMALIZE);
	
    // material
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  m_emission);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   m_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   m_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  m_specular);
	glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, m_shininess);
	
    // lighting
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, l_ambient );
	glLightModeli ( GL_LIGHT_MODEL_LOCAL_VIEWER, l_localViewer );
	glLightModeli ( GL_LIGHT_MODEL_TWO_SIDE, l_twoSides );
	glLightModeli ( GL_LIGHT_MODEL_COLOR_CONTROL, l_separateDiffuseAndSpecular );

    // light 0
	glLightfv( GL_LIGHT0, GL_AMBIENT,   l0_ambient );
	glLightfv( GL_LIGHT0, GL_DIFFUSE,   l0_diffuse );
	glLightfv( GL_LIGHT0, GL_SPECULAR,  l0_specular );

    // light 1
	glLightfv( GL_LIGHT1, GL_AMBIENT,   l1_ambient );
	glLightfv( GL_LIGHT1, GL_DIFFUSE,   l1_diffuse );
	glLightfv( GL_LIGHT1, GL_SPECULAR,  l1_specular );

    // clear all
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glClear( GL_COLOR_BUFFER_BIT  | GL_DEPTH_BUFFER_BIT); // la couleur et le z
	glLoadIdentity();  // repere camera 

    // draw
    glLightfv( GL_LIGHT0, GL_POSITION,  l0_position ); // coords in camera frame
	
    tbVisuTransform(); // viewing transform
	
	if(lumiere1){
		glPushMatrix(); 
		glRotatef( longitude1, 0,1,0 );
        glRotatef( lattitude1, 1,0,0 );
        glTranslatef( 0,0,distance1 );
        glLightfv( GL_LIGHT1, GL_POSITION,  l1_position );  //  coords in mobile frame
        glColor3f(1,1,1);
        glutWireCube(0.1);
        glPopMatrix();
	}
	
	
    glScalef(1,1,1/zMax);  // flatten the function

    // draw
	for( int i=0; i<w_img-1; i++ )
	{
		glDrawElements(GL_TRIANGLE_STRIP,2*h_img, GL_UNSIGNED_INT,indices+i*2*h_img);
	}

	glutSwapBuffers();
}

void reshape(int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective (50, (float)w/h, 1, 10);
   glMatrixMode(GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
	//printf("key %d pressed at %d,%d\n",key,x,y); fflush(stdout);
	switch (key) {
		case 27:     // touche ESC
			exit(0);
		case '0':
			lumiere0 = !lumiere0;
			glutPostRedisplay();
			break;
		case '1':
			lumiere1 = !lumiere1;
			glutPostRedisplay();
			break;
	}
}

void specialKeys(int key, int x, int y)
{
	//printf("specialkey %d pressed at %d,%d\n",key,x,y); fflush(stdout);
	switch (key) {
        case GLUT_KEY_LEFT:
			longitude1 = longitude1 -1;
			glutPostRedisplay();
			break;
        case GLUT_KEY_RIGHT:
			longitude1 = longitude1 + 1;
			glutPostRedisplay();
			break;
        case GLUT_KEY_UP:
            lattitude1 = lattitude1 -1;
			glutPostRedisplay();
			break;
        case GLUT_KEY_DOWN:
            lattitude1 = lattitude1 +1;
			glutPostRedisplay();
			break;
	}
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);

    // framebuffer layers used in this application
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );

    // window
	glutInitWindowPosition(200, 100);
	glutInitWindowSize(W_fen,H_fen);
	glutCreateWindow(argv[0]);

    // init
    computeGeometry();

    // Init the trackball
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0,0,-4);   
    tbInitTransform();     // initial viewpoint

    cout<<"Moving the viewpoint:" << endl;
    tbHelp();              // display help

    cout<<"Switching ligth 0 on/off: key 0"<<endl;
    cout<<"Switching ligth 1 on/off: key 1"<<endl;
    cout<<"Moving light 1: use the arrows"<<endl;


    // callbacks
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialKeys);
	glutDisplayFunc(display);
    glutMouseFunc(tbMouseFunc);    // traqueball uses the mouse
    glutMotionFunc(tbMotionFunc);  // traqueboule uses the mouse

    //
	glutMainLoop();
    return 0;  // we never reach this point
}
